@@ -145,18 +145,24 @@ class TestAnnotationRels:
         with pytest.raises(Exception) as ex:
             Annotation(3, 2, "X")
         assert str(ex.value).startswith("Cannot create annotation")
-        with pytest.raises(Exception) as ex:
+        with pytest.raises(Exception):
             Annotation(1, 2, "X", annid="x")
-        with pytest.raises(Exception) as ex:
+        with pytest.raises(Exception):
             Annotation(1, 2, "X", features=12)
-        with pytest.raises(Exception) as ex:
+        with pytest.raises(Exception):
             Annotation(1, 2, "X", annid="x")
         assert Annotation(2, 3, "X").span == Span(2, 3)
         assert not (Annotation(1, 2, "X") == "X")
         ann1 = Annotation(1, 2, "X")
+        ann2 = Annotation(1, 2, "X")
         assert ann1 == ann1
-        assert hash(ann1) == hash((ann1.id, ann1._owner_set))
-        with pytest.raises(Exception) as ex:
+        assert ann1 != ann2
+        assert ann1.same(ann2)
+        assert ann1.equal(ann2)
+        ann3 = Annotation(1, 2, "X", annid=22)
+        assert ann3.equal(ann1)
+        assert not ann3.same(ann1)
+        with pytest.raises(Exception):
             Annotation(1, 2, "X") < 33
         assert Annotation(1, 2, "X") < Annotation(2, 3, "X")
         assert not Annotation(1, 2, "X") < Annotation(0, 3, "X")
@@ -170,7 +176,7 @@ class TestAnnotationRels:
         ann1 = Annotation(1, 2, "x", dict(a=1), annid=3)
         dict1 = ann1.to_dict()
         ann2 = Annotation.from_dict(dict1)
-        assert ann1 == ann2
+        assert ann1.equal(ann2)
 
         assert ann2.features == {"a": 1}
 
