@@ -1,6 +1,6 @@
-import sys
+
 from gatenlp import Document, Annotation, Span
-from gatenlp.pam.pampac import Context, Location
+from gatenlp.pam.pampac import Context, Location, Result
 
 from gatenlp.pam.pampac import (
     Ann,
@@ -14,9 +14,15 @@ from gatenlp.pam.pampac import (
     Success,
     Failure,
     Rule,
-    Pampac
+    Pampac,
+    Function
 )
 
+# Disable:
+#   too-many-statements (R0915)
+#   import-outside-toplevel (C0415)
+# pylint: disable=R0915, C0415
+
 
 class TestPampac01:
 
@@ -40,7 +46,7 @@ class TestPampac01:
         loc = ret[0].location
         assert loc.text_location == 2
         assert loc.ann_location == 1
-        assert len(ret[0].data) == 1
+        assert len(ret[0].matches) == 1
 
         # do this with the match method
         ret = parser(doc, annlist)
@@ -49,7 +55,7 @@ class TestPampac01:
         loc = ret[0].location
         assert loc.text_location == 2
         assert loc.ann_location == 1
-        assert len(ret[0].data) == 1
+        assert len(ret[0].matches) == 1
 
         # this does NOT first advance the annotation index so the annotation start index
         # is at least 2. So it matches the annotation at index 1 which ends at 1 which is
@@ -59,7 +65,7 @@ class TestPampac01:
         assert len(ret) == 1
         loc = ret[0].location
         assert loc == Location(1, 2)
-        assert len(ret[0].data) == 1
+        assert len(ret[0].matches) == 1
 
         # by default we do advance, so we match the last annotation and end up at text
         # position 4 looking for annotation index 5
@@ -68,45 +74,45 @@ class TestPampac01:
         assert len(ret) == 1
         loc = ret[0].location
         assert loc == Location(3, 5)
-        assert len(ret[0].data) == 1
+        assert len(ret[0].matches) == 1
 
         # Try to fail
         parser = Ann("Token")
         ret = parser(doc, annlist)
         assert isinstance(ret, Failure)
 
-        # Same without a name: should generate the same locations, but no data
+        # Same without a name: should generate the same locations, but no matches
         parser = Ann()
         ret = parser.parse(Location(), ctx)
         assert len(ret) == 1
         loc = ret[0].location
         assert loc.text_location == 2
         assert loc.ann_location == 1
-        assert len(ret[0].data) == 0
+        assert len(ret[0].matches) == 0
 
         ret = Ann().parse(loc, ctx)
         assert len(ret) == 1
         loc = ret[0].location
         assert loc.text_location == 3
         assert loc.ann_location == 5
-        assert len(ret[0].data) == 0
+        assert len(ret[0].matches) == 0
 
         parser = AnnAt(name="a2")
         ret = parser.parse(Location(), ctx)
         assert len(ret) == 1
-        assert len(ret[0].data) == 1
+        assert len(ret[0].matches) == 1
 
         parser = AnnAt(matchtype="all", name="a3")
         ret = parser.parse(Location(), ctx)
         assert len(ret) == 2
-        assert len(ret[0].data) == 1
-        assert len(ret[1].data) == 1
+        assert len(ret[0].matches) == 1
+        assert len(ret[1].matches) == 1
 
         # Try Rule
         parser = Ann(name="a1")
         tmp = dict(i=0)
 
-        def rhs1(succ, **kwargs):
+        def rhs1(_succ, **_kwargs):
             tmp["i"] = 1
 
         rule = Call(parser, rhs1)
@@ -115,11 +121,11 @@ class TestPampac01:
         loc = ret[0].location
         assert loc.text_location == 2
         assert loc.ann_location == 1
-        assert len(ret[0].data) == 1
+        assert len(ret[0].matches) == 1
         assert tmp["i"] == 1
 
         # use the call method instead
-        def rhs2(succ, **kwargs):
+        def rhs2(_succ, **_kwargs):
             tmp["i"] = 2
 
         parser = Ann(name="a1").call(rhs2)
@@ -191,17 +197,17 @@ class TestPampac01:
         ret = Seq(AnnAt("Token", name="1"), AnnAt("Ann", name="2")).match(doc, annlist)
         assert ret.issuccess()
         assert len(ret) == 1
-        assert len(ret[0].data) == 2
-        assert ret[0].data[0]["ann"].id == 2
-        assert ret[0].data[1]["ann"].id == 3
+        assert len(ret[0].matches) == 2
+        assert ret[0].matches[0]["ann"].id == 2
+        assert ret[0].matches[1]["ann"].id == 3
 
         # match sequence Ann/Ann, take first at each point
         ret = Seq(AnnAt("Ann", name="1"), AnnAt("Ann", name="2")).match(doc, annlist)
         assert ret.issuccess()
         assert len(ret) == 1
-        assert len(ret[0].data) == 2
-        assert ret[0].data[0]["ann"].id == 0
-        assert ret[0].data[1]["ann"].id == 3
+        assert len(ret[0].matches) == 2
+        assert ret[0].matches[0]["ann"].id == 0
+        assert ret[0].matches[1]["ann"].id == 3
 
         # match sequence Ann/Ann, take first at each point, set useoffset=False so we do not skip to the
         # end offset of the previous before matching the next
@@ -211,24 +217,24 @@ class TestPampac01:
         ).match(doc, annlist)
         assert ret.issuccess()
         assert len(ret) == 1
-        assert len(ret[0].data) == 2
-        assert ret[0].data[0]["ann"].id == 0
-        assert ret[0].data[1]["ann"].id == 1
+        assert len(ret[0].matches) == 2
+        assert ret[0].matches[0]["ann"].id == 0
+        assert ret[0].matches[1]["ann"].id == 1
 
         # Make sure we get the correct set of annotations at position 0 and 2
         ret = AnnAt("Ann", name="a", matchtype="all").match(doc, annlist)
         assert ret.issuccess()
         assert len(ret) == 2
-        assert ret[0].data[0]["ann"].id == 0
-        assert ret[1].data[0]["ann"].id == 1
+        assert ret[0].matches[0]["ann"].id == 0
+        assert ret[1].matches[0]["ann"].id == 1
         # ret.pprint()
         ret = AnnAt("Ann", name="a", matchtype="all").match(
             doc, annlist, location=Location(2, 2)
         )
         assert ret.issuccess()
         assert len(ret) == 2
-        assert ret[0].data[0]["ann"].id == 3
-        assert ret[1].data[0]["ann"].id == 4
+        assert ret[0].matches[0]["ann"].id == 3
+        assert ret[1].matches[0]["ann"].id == 4
         # ret.pprint()
 
         # Match sequence of two anns in order, take all results
@@ -240,18 +246,18 @@ class TestPampac01:
         ).match(doc, annlist)
         assert ret.issuccess()
         assert len(ret) == 4
-        assert len(ret[0].data) == 2
-        assert len(ret[1].data) == 2
-        assert len(ret[2].data) == 2
-        assert len(ret[3].data) == 2
-        assert ret[0].data[0]["ann"].id == 0
-        assert ret[0].data[1]["ann"].id == 3
-        assert ret[1].data[0]["ann"].id == 0
-        assert ret[1].data[1]["ann"].id == 4
-        assert ret[2].data[0]["ann"].id == 1
-        assert ret[2].data[1]["ann"].id == 3
-        assert ret[3].data[0]["ann"].id == 1
-        assert ret[3].data[1]["ann"].id == 4
+        assert len(ret[0].matches) == 2
+        assert len(ret[1].matches) == 2
+        assert len(ret[2].matches) == 2
+        assert len(ret[3].matches) == 2
+        assert ret[0].matches[0]["ann"].id == 0
+        assert ret[0].matches[1]["ann"].id == 3
+        assert ret[1].matches[0]["ann"].id == 0
+        assert ret[1].matches[1]["ann"].id == 4
+        assert ret[2].matches[0]["ann"].id == 1
+        assert ret[2].matches[1]["ann"].id == 3
+        assert ret[3].matches[0]["ann"].id == 1
+        assert ret[3].matches[1]["ann"].id == 4
 
     def test03(self):
         """
@@ -284,12 +290,12 @@ class TestPampac01:
         ).match(doc, annlist)
         assert ret.issuccess()
         assert len(ret) == 1
-        assert len(ret[0].data) == 3
-        assert ret[0].data[0]["ann"].id == 0
-        assert ret[0].data[1]["ann"].id == 3
-        assert ret[0].data[2]["ann"].id == 5
+        assert len(ret[0].matches) == 3
+        assert ret[0].matches[0]["ann"].id == 0
+        assert ret[0].matches[1]["ann"].id == 3
+        assert ret[0].matches[2]["ann"].id == 5
 
-        # Same as before, but with a name, so we should get one additional data for the whole sequence
+        # Same as before, but with a name, so we should get one additional matches for the whole sequence
         # with a span
         ret = N(
             AnnAt("Ann", name="a1", matchtype="first"),
@@ -301,11 +307,11 @@ class TestPampac01:
         ).match(doc, annlist)
         assert ret.issuccess()
         assert len(ret) == 1
-        assert len(ret[0].data) == 4
-        assert ret[0].data[0]["ann"].id == 0
-        assert ret[0].data[1]["ann"].id == 3
-        assert ret[0].data[2]["ann"].id == 5
-        assert ret[0].data[3]["span"] == Span(0, 6)
+        assert len(ret[0].matches) == 4
+        assert ret[0].matches[0]["ann"].id == 0
+        assert ret[0].matches[1]["ann"].id == 3
+        assert ret[0].matches[2]["ann"].id == 5
+        assert ret[0].matches[3]["span"] == Span(0, 6)
 
         # single Ann, single result from N
         # this should return annotation ids 0, 3, 5, 8
@@ -318,11 +324,11 @@ class TestPampac01:
         ).match(doc, annlist)
         assert ret.issuccess()
         assert len(ret) == 1
-        assert len(ret[0].data) == 4
-        assert ret[0].data[0]["ann"].id == 0
-        assert ret[0].data[1]["ann"].id == 3
-        assert ret[0].data[2]["ann"].id == 5
-        assert ret[0].data[3]["ann"].id == 8
+        assert len(ret[0].matches) == 4
+        assert ret[0].matches[0]["ann"].id == 0
+        assert ret[0].matches[1]["ann"].id == 3
+        assert ret[0].matches[2]["ann"].id == 5
+        assert ret[0].matches[3]["ann"].id == 8
 
         # single Ann, single result from N, with early stopping at Person
         # this should return annotation ids 0, 3, 7
@@ -336,14 +342,14 @@ class TestPampac01:
         ).match(doc, annlist)
         assert ret.issuccess()
         assert len(ret) == 1
-        assert len(ret[0].data) == 3
-        assert ret[0].data[0]["ann"].id == 0
-        assert ret[0].data[1]["ann"].id == 3
-        assert ret[0].data[2]["ann"].id == 7
+        assert len(ret[0].matches) == 3
+        assert ret[0].matches[0]["ann"].id == 0
+        assert ret[0].matches[1]["ann"].id == 3
+        assert ret[0].matches[2]["ann"].id == 7
 
         # Try a match with min=0 and max=99 that does not succeed
         # single Ann, single result from N
-        # this should return an empty list for data
+        # this should return an empty list for matches
         ret = N(
             AnnAt("NotThere", name="a1", matchtype="first"),
             min=0,
@@ -353,11 +359,11 @@ class TestPampac01:
         ).match(doc, annlist)
         assert ret.issuccess()
         assert len(ret) == 1
-        assert len(ret[0].data) == 0
+        assert len(ret[0].matches) == 0
 
         # Try a match with min=0 and max=99 that does not succeed
         # single Ann, single result from N
-        # this should return an empty list for data
+        # this should return an empty list for matches
         ret = N(
             AnnAt("Ann", name="a1", matchtype="first"),
             min=0,
@@ -367,11 +373,11 @@ class TestPampac01:
         ).match(doc, annlist)
         assert ret.issuccess()
         assert len(ret) == 1
-        assert len(ret[0].data) == 4
-        assert ret[0].data[0]["ann"].id == 0
-        assert ret[0].data[1]["ann"].id == 3
-        assert ret[0].data[2]["ann"].id == 5
-        assert ret[0].data[3]["ann"].id == 8
+        assert len(ret[0].matches) == 4
+        assert ret[0].matches[0]["ann"].id == 0
+        assert ret[0].matches[1]["ann"].id == 3
+        assert ret[0].matches[2]["ann"].id == 5
+        assert ret[0].matches[3]["ann"].id == 8
 
     def test04(self):
         """
@@ -404,10 +410,10 @@ class TestPampac01:
         ).match(doc, annlist)
         assert ret.issuccess()
         assert len(ret) == 1
-        assert len(ret[0].data) == 3
-        assert ret[0].data[0]["ann"].id == 0
-        assert ret[0].data[1]["ann"].id == 3
-        assert ret[0].data[2]["ann"].id == 5
+        assert len(ret[0].matches) == 3
+        assert ret[0].matches[0]["ann"].id == 0
+        assert ret[0].matches[1]["ann"].id == 3
+        assert ret[0].matches[2]["ann"].id == 5
 
         # multiple Anns, all results from N
         # should return 0,1
@@ -420,10 +426,10 @@ class TestPampac01:
         ).match(doc, annlist)
         assert ret.issuccess()
         assert len(ret) == 2
-        assert len(ret[0].data) == 1
-        assert len(ret[1].data) == 1
-        assert ret[0].data[0]["ann"].id == 0
-        assert ret[1].data[0]["ann"].id == 1
+        assert len(ret[0].matches) == 1
+        assert len(ret[1].matches) == 1
+        assert ret[0].matches[0]["ann"].id == 0
+        assert ret[1].matches[0]["ann"].id == 1
 
         # multiple Anns, all results from N
         ret = N(
@@ -435,18 +441,18 @@ class TestPampac01:
         ).match(doc, annlist)
         assert ret.issuccess()
         assert len(ret) == 4
-        assert len(ret[0].data) == 2
-        assert len(ret[1].data) == 2
-        assert len(ret[2].data) == 2
-        assert len(ret[3].data) == 2
-        assert ret[0].data[0]["ann"].id == 0
-        assert ret[0].data[1]["ann"].id == 3
-        assert ret[1].data[0]["ann"].id == 0
-        assert ret[1].data[1]["ann"].id == 4
-        assert ret[2].data[0]["ann"].id == 1
-        assert ret[2].data[1]["ann"].id == 3
-        assert ret[3].data[0]["ann"].id == 1
-        assert ret[3].data[1]["ann"].id == 4
+        assert len(ret[0].matches) == 2
+        assert len(ret[1].matches) == 2
+        assert len(ret[2].matches) == 2
+        assert len(ret[3].matches) == 2
+        assert ret[0].matches[0]["ann"].id == 0
+        assert ret[0].matches[1]["ann"].id == 3
+        assert ret[1].matches[0]["ann"].id == 0
+        assert ret[1].matches[1]["ann"].id == 4
+        assert ret[2].matches[0]["ann"].id == 1
+        assert ret[2].matches[1]["ann"].id == 3
+        assert ret[3].matches[0]["ann"].id == 1
+        assert ret[3].matches[1]["ann"].id == 4
 
         # multiple Anns, all results from N
         # just three for the first ann: 0,1,2
@@ -459,12 +465,12 @@ class TestPampac01:
         ).match(doc, annlist)
         assert ret.issuccess()
         assert len(ret) == 3
-        assert len(ret[0].data) == 1
-        assert len(ret[1].data) == 1
-        assert len(ret[2].data) == 1
-        assert ret[0].data[0]["ann"].id == 0
-        assert ret[1].data[0]["ann"].id == 1
-        assert ret[2].data[0]["ann"].id == 2
+        assert len(ret[0].matches) == 1
+        assert len(ret[1].matches) == 1
+        assert len(ret[2].matches) == 1
+        assert ret[0].matches[0]["ann"].id == 0
+        assert ret[1].matches[0]["ann"].id == 1
+        assert ret[2].matches[0]["ann"].id == 2
 
         # This should just find the Token as the first and only match!
         ret = N(AnnAt("Ann", name="a1", matchtype="all"),
@@ -476,9 +482,8 @@ class TestPampac01:
                 ).match(doc, annlist)
         assert ret.issuccess()
         assert len(ret) == 1
-        assert len(ret[0].data) == 1
-        assert ret[0].data[0]["ann"].id == 2
-
+        assert len(ret[0].matches) == 1
+        assert ret[0].matches[0]["ann"].id == 2
 
         # This should terminate with Person and find all paths that can lead up to PErson:
         # 0,3 0,4 1,3 1,4
@@ -491,22 +496,22 @@ class TestPampac01:
                 ).match(doc, annlist)
         assert ret.issuccess()
         assert len(ret) == 4
-        assert len(ret[0].data) == 3
-        assert len(ret[1].data) == 3
-        assert len(ret[2].data) == 3
-        assert len(ret[3].data) == 3
-        assert ret[0].data[0]["ann"].id == 0
-        assert ret[0].data[1]["ann"].id == 3
-        assert ret[0].data[2]["ann"].id == 7
-        assert ret[1].data[0]["ann"].id == 0
-        assert ret[1].data[1]["ann"].id == 4
-        assert ret[1].data[2]["ann"].id == 7
-        assert ret[2].data[0]["ann"].id == 1
-        assert ret[2].data[1]["ann"].id == 3
-        assert ret[2].data[2]["ann"].id == 7
-        assert ret[3].data[0]["ann"].id == 1
-        assert ret[3].data[1]["ann"].id == 4
-        assert ret[3].data[2]["ann"].id == 7
+        assert len(ret[0].matches) == 3
+        assert len(ret[1].matches) == 3
+        assert len(ret[2].matches) == 3
+        assert len(ret[3].matches) == 3
+        assert ret[0].matches[0]["ann"].id == 0
+        assert ret[0].matches[1]["ann"].id == 3
+        assert ret[0].matches[2]["ann"].id == 7
+        assert ret[1].matches[0]["ann"].id == 0
+        assert ret[1].matches[1]["ann"].id == 4
+        assert ret[1].matches[2]["ann"].id == 7
+        assert ret[2].matches[0]["ann"].id == 1
+        assert ret[2].matches[1]["ann"].id == 3
+        assert ret[2].matches[2]["ann"].id == 7
+        assert ret[3].matches[0]["ann"].id == 1
+        assert ret[3].matches[1]["ann"].id == 4
+        assert ret[3].matches[2]["ann"].id == 7
 
     def test05(self):
         """
@@ -527,10 +532,10 @@ class TestPampac01:
         # first make sure the pattern works as we want
         ctx = Context(doc=doc, anns=annlist)
         pat1 = AnnAt("Ann2", name="a1") >> AnnAt("Ann2", name="a2")
-        loc = ctx.inc_location(Location(0,0), by_offset=1)
-        ret = pat1.parse(location=loc, context=ctx)
+        loc = ctx.inc_location(Location(0, 0), by_offset=1)
+        pat1.parse(location=loc, context=ctx)
 
-        def r1_action(succ, context=None, **kwargs):
+        def r1_action(succ, _context=None, **_kwargs):
             span = succ[0].span
             ann = succ.context.outset.add(span.start, span.end, "NEW")
             return ann
@@ -554,3 +559,115 @@ class TestPampac01:
         assert a.end == 5
         assert a.type == "NEW"
         assert len(outset) == orig_len + 1
+
+
+class TestPampacRemoveAnn:
+
+    def test01(self):
+        """
+        Unit test method (make linter happy)
+        """
+        from gatenlp.pam.pampac.actions import RemoveAnn
+
+        def make_doc():
+            doc = Document("Some test document")
+            doc.annset().add(0, 2, "Ann")  # 0
+            doc.annset().add(0, 2, "Ann")  # 1
+            doc.annset().add(0, 2, "Token")  # 2
+            doc.annset().add(2, 4, "Ann")  # 3
+            doc.annset().add(2, 4, "Ann")  # 4
+            doc.annset().add(4, 6, "Ann")  # 5
+            doc.annset().add(4, 6, "Ann")  # 6
+            doc.annset().add(4, 6, "Person")  # 7
+            doc.annset().add(6, 8, "Ann")  # 8
+            doc.annset().add(6, 8, "Ann")  # 9
+            doc.annset().add(8, 10, "XXXX")  # 10
+            return doc
+
+        doc = make_doc()
+
+        # match first match of Ann an remove
+        assert len(doc.annset()) == 11
+        Pampac(
+            Rule(
+                AnnAt("Ann", name="match", matchtype="all"),
+                RemoveAnn("match", doc.annset())
+            ),
+        ).run(doc, doc.annset().with_type("Ann", "XXX", "Person", "Token"))
+        assert len(doc.annset()) == 7
+
+
+class TestPampacMisc:
+
+    def test01(self):
+        """
+        Unit test method (make linter happy)
+        """
+        loc1 = Location(0, 0)
+        loc2 = Location(0, 1)
+        loc3 = Location(1, 0)
+        loc4 = Location(0, 0)
+
+        assert loc1 != loc2
+        assert loc1 != loc3
+        assert loc1 == loc4
+        assert loc1 != "asa"
+
+        assert str(loc1) == "Location(0,0)"
+
+        res1 = Result(location=Location(10, 10), span=Span(4, 10))
+        assert list(res1.anns4matches()) == []
+        assert res1.matches4name("xxx") == []
+
+        res2 = Result(matches={"span": Span(3, 4), "name": "xx"}, location=Location(12, 12), span=Span(4, 12))
+        assert list(res2.anns4matches()) == []
+        assert res2.matches4name("xx") == [{"span": Span(3, 4), "name": "xx"}]
+
+        res3 = Result(matches=[{"span": Span(3, 4), "name": "xx"}, {"span": Span(3, 4), "name": "yy"}, ],
+                      location=Location(10, 10), span=Span(4, 10))
+        assert list(res3.anns4matches()) == []
+        assert res3.matches4name("xx") == [{"span": Span(3, 4), "name": "xx"}]
+
+        assert str(res1) == "Result(loc=Location(10,10),span=Span(4,10),nmatches=0)"
+        assert res1.__repr__() == "Result(loc=Location(10,10),span=Span(4,10),matches=[])"
+
+        fail1 = Failure()
+        assert not fail1.issuccess()
+        assert fail1.describe() == """None at ?/?: Parser Error"""
+
+        fail2 = Failure(message="Some problem", parser="Parser1", location=loc1)
+        assert fail2.describe() == """Parser1 at 0/0: Some problem"""
+
+        fail3 = Failure(message="Another problem", parser="Parser2", location=loc1, causes=[fail2, fail1])
+        assert fail3.describe() == """Parser2 at 0/0: Another problem
+Caused by:
+    Parser1 at 0/0: Some problem
+    None at ?/?: Parser Error"""
+        #
+        doc1 = Document("somedoc")
+        set1 = doc1.annset()
+        ctx1 = Context(doc1, set1)
+        assert ctx1.annset.size == 0
+        assert ctx1.get_ann(loc1) is None
+
+        succ1 = Success(res1, ctx1)
+        assert succ1.issuccess()
+        assert succ1.select_result([res1, res2]) == res1
+        assert succ1.select_result([res1, res2], matchtype="all") == [res1, res2]
+        assert succ1.select_result([res1, res2], matchtype="longest") == res2
+        assert succ1.select_result([res1, res2], matchtype="shortest") == res1
+        assert succ1.result(matchtype="first") == res1
+        for r in succ1:
+            assert isinstance(r, Result)
+        assert succ1[0] == res1
+
+    def test02(self):
+        """
+        Unit test method (make linter happy)
+        """
+
+        def fun1(location, context):
+            return location, context
+
+        parser1 = Function(fun1)
+        assert parser1.parse(1, 2) == (1, 2)
